#ifndef util_CLIPBOARD_HPP_INCLUDED
#define util_CLIPBOARD_HPP_INCLUDED 1

#include <s11n.net/s11n/data_node.hpp>
#include <s11n.net/s11n/data_node_serialize.hpp>
#include <s11n.net/s11n/traits.hpp>
#include <s11n.net/sigslot/sigslot.hpp> // signals/slots framework.

#define CB_DEBUG 0
#if CB_DEBUG
#  include <s11n.net/s11n/s11n_debuggering_macros.hpp> // CERR
#  include <iostream>
#endif
namespace s11n { namespace cb {

#if CB_DEBUG
        /**
           A debuggering type which shows when a clipboard is updated.
           Only enabled if CB_DEBUG is 1.
        */
        template <typename CBT>
        struct cb_debuggerer : public sigslot::has_slots
        {
                typedef typename CBT::node_type NT;
                void slot_updated( const CBT *cb, const NT *n )
                {
                        CERR << "clipboard @ " << std::hex << cb << " updated: node @ " << n << "\n";
                }
        };
#endif // debug

        /**
           Experimental clipboard area for s11n data nodes.
        */
        template <typename NodeT, typename ContextT = NodeT>
        class clipboard : public sigslot::has_slots
        {
        public:
                typedef clipboard<NodeT,ContextT> this_type;
                typedef NodeT node_type;
                typedef s11n::node_traits<NodeT> traits_type;
                typedef ContextT context_type;
#if CB_DEBUG
                typedef cb_debuggerer<this_type> DbgT;
                mutable DbgT m_dbg;
#endif // debug


                /**
                   Clients can connect slots to this to learn when
                   this object's content is updated.
                */
                sigslot::signal2<const this_type *,const node_type *> signal_content_updated;

                clipboard() : m_owns(false), m_c(0)
                {
#if CB_DEBUG
                        this->signal_content_updated.connect_slot( &(this->m_dbg), &DbgT::slot_updated );
#endif
                }

                ~clipboard()
                {
                        this->free();
                }

                /**
                   Tries to serialize the given object using s11n,
                   with SerializableT as the base Serializable type.
                   Returns true on success, false on error. This
                   object owns the serialized copy.
                */
                template <typename SerializableT>
                bool serialize( const SerializableT & c )
                {
                        this->free();
                        if( ! c ) return;
                        
                        this->m_c = traits_type::create( "clipboard_data" );
                        this->m_owns = true;
                        if( ! s11n::serialize( *(this->m_c), c ) )
                        {
                                this->free();
                        }
                        this->updated();
                        return this->m_owns;
                }

                /**
                   Tries to deserialize the current content using
                   s11n, with DeserializableT as the base Serializable
                   type. Caller owns the returns pointer, which may be
                   0 or may be a subtype of DeserializableT.
                */
                template <typename DeserializableT>
                DeserializableT * deserialize() const
                {
                        if( ! this->m_c ) return 0;
                        return s11n::deserialize<node_type,DeserializableT>( *(this->m_c) );
                }

                /**
                   Returns a copy of the current content, or 0 if
                   there is none.
                */
                node_type * copy() const
                {
                        return (!this->m_c)
                                ? 0
                                : new node_type(*this->m_c);
                }

                /**
                   Copies n to the clipboard.
                */
                void copy( const node_type & n )
                {
                        this->free();
                        this->m_c = new node_type(n);
                        this->m_owns = true;
                        this->updated();
                }

                /**
                   "Cuts" n to the clipboard. This object takes
                   ownership of n.
                */
                void cut( node_type * n )
                {
                        this->free();
                        this->m_c = n;
                        this->m_owns = true;
                        this->updated();
                }


                /**
                   Current content. May be 0. Note the constness. If
                   you delete the returned pointer, results are
                   undefined.
                */
                const node_type * content() const
                {
                        return this->m_c;
                }

                /**
                   Caller takes ownership of clipboard content.
                */
                node_type * take()
                {
                        this->m_owns = false;
                        return this->m_c;
                }

                /**
                   Frees any resources held by this clipboard.  If the
                   clipboard does not actually own it's content then
                   it does not delete it. Normally the clipboard owns
                   it's content object unless take() is called.

                   After calling this, content() will return 0.
                */
                void free()
                {
                        if( this->m_owns && this->m_c )
                        {
                                delete this->m_c;
                                this->m_c = 0;
                                this->updated();
                        }
                        this->m_owns = false;
                }

                /**
                   Returns a shared instance of this type.
                */
                static this_type & shared()
                {
                        static this_type bob;
                        return bob;
                }
                

        private:
                void updated()
                {
                        this->signal_content_updated( this, this->m_c );
                }

                bool m_owns;
                node_type * m_c;
        };

} } // namespace s11n::cb

#undef CB_DEBUG

#endif // util_CLIPBOARD_HPP_INCLUDED
